package com.jkcredit.traffic.record.service;

import com.alibaba.fastjson.JSON;
import com.jkcredit.traffic.record.constant.CommonConstant;
import com.jkcredit.traffic.record.model.CommonResponseObject;
import com.jkcredit.traffic.record.model.MonthResult;
import com.jkcredit.traffic.record.test.AsyncPushTask;
import com.jkcredit.traffic.record.util.CsvUtil;
import com.jkcredit.traffic.record.util.FileUtil;
import lombok.extern.slf4j.Slf4j;
import org.apache.spark.sql.SparkSession;
import org.springframework.beans.factory.annotation.Autowired;
import org.springframework.stereotype.Service;

import java.io.File;
import java.util.ArrayList;
import java.util.List;

/**
 * @author xusonglin
 * @version V1.0
 **/
@Service
@Slf4j
public class TrafficRecordsAnalyseServiceImpl implements TrafficRecordsAnalyseService {
    @Autowired
    SparkSession sparkSession;
    @Autowired
    AsyncPushTask asyncPushTask;

    @Override
    public CommonResponseObject pushTrafficRecords(String filePath) {
        List<File> files = new ArrayList<>();
        File folder = new File(filePath);
        File[] tempList = folder.listFiles();
        if (tempList.length == 0) {
            return new CommonResponseObject(CommonConstant.ERROR_CODE_FILE_NOT_EXIST,
                    CommonConstant.ERROR_MESSAGE_FILE_NOT_EXIST);
        }

        for (int i = 0; i < tempList.length; i++) {
            if (tempList[i].isFile() && tempList[i].getName().split("\\.")[1].equals("csv")) {
                files.add(tempList[i]);
            }
        }

        for (int i = 0; i < files.size(); i++) {
            String fileName = folder.getName() + "_" + i;
            asyncPushTask.push(fileName, files.get(i).getAbsolutePath());
        }

        return new CommonResponseObject().success();
    }

    @Override
    public CommonResponseObject pushErrorRecords(String filePath) {
        List<File> files = new ArrayList<>();
        File folder = new File(filePath);
        File[] tempList = folder.listFiles();
        if (tempList.length == 0) {
            return new CommonResponseObject(CommonConstant.ERROR_CODE_FILE_NOT_EXIST,
                    CommonConstant.ERROR_MESSAGE_FILE_NOT_EXIST);
        }

        for (int i = 0; i < tempList.length; i++) {
            if (tempList[i].isFile() && tempList[i].getName().split("\\.")[1].equals("csv")) {
                files.add(tempList[i]);
            }
        }

        for (int i = 0; i < files.size(); i++) {
            List<String> errorRecords = CsvUtil.readCsv(files.get(i).getAbsolutePath());
            List<MonthResult> monthResults = new ArrayList<>();
            for (String record : errorRecords) {
                MonthResult monthResult = JSON.toJavaObject(JSON.parseObject(record), MonthResult.class);
                monthResults.add(monthResult);
            }
            try {
                asyncPushTask.pushRecords(monthResults);
            } catch (Exception e) {
                log.error("推送异常数据失败:", e);
                return new CommonResponseObject().failed();
            }
        }
        return new CommonResponseObject().success();
    }
}
