package com.jkcredit.traffic.record.test;

import com.ciphergateway.ciphersuite.*;

import javax.xml.bind.DatatypeConverter;
import java.util.ArrayList;
import java.util.List;
import java.util.Scanner;

/**
 * @ClassName : CipherSuiteUtilsTest
 * @Description : 密码套件工具测试
 * @Author : bcx
 * @Date: 2020-05-13 14:03
 */
public class CipherSuiteUtilsTest {


    public static void main(String[] args) throws Exception {
        Scanner scanner = new Scanner(System.in);
        List<String> algorithmList = new ArrayList<String>();
        for (AlgorithmEnum algorithmEnum : AlgorithmEnum.values()) {
            algorithmList.add(algorithmEnum.name());
        }
        System.out.println("请选择算法：");
        int index = 1;
        for (String algorithm : algorithmList) {
            System.out.println("【" + (index++) + "】 " + algorithm);
        }


        int algorithmIndex = scanner.nextInt();
        if (algorithmIndex > algorithmList.size() || algorithmIndex < 1) {
            System.out.println("算法序号不合法");
            return;
        }
        String algorithm = algorithmList.get(algorithmIndex - 1);

        System.out.println("请输入密钥索引：");
        String keyId = scanner.next();

        System.out.println("algorithm: " + algorithm);
        System.out.println("keyId: " + keyId);


        String plainStr = null;
        int precision = 5;
        int scale = 2;

        if ("SM4_ECB".equals(algorithm) || "SM4_ECB_V2".equals(algorithm) || "SM4_CTR".equals(algorithm) || "SM4_GCM".equals(algorithm) || "SM4_CBC".equals(algorithm) || "SM3".equals(algorithm) || "SM2".equals(algorithm) || "AES_ECB".equals(algorithm) || "AES_CTR".equals(algorithm) || "AES_GCM".equals(algorithm) || "AES_CBC".equals(algorithm) || "RSA".equals(algorithm)) {
            plainStr = "Hello World";
        } else if ("FPE_EMAIL".equals(algorithm)) {
            plainStr = "hello@163.com";
        } else if ("FPE_PHONE_NUM".equals(algorithm)) {
            plainStr = "13581690000";
        } else if ("FPE_OFFICER_CARD".equals(algorithm)) {
            plainStr = "军字第00111206号";
        } else if ("FPE_ID_CARD".equals(algorithm)) {
            plainStr = "110103201206200910";
        } else if ("FPE_PASSPORT".equals(algorithm)) {
            plainStr = "E12345678";
        } else if ("FPE_IDENTIFICATION".equals(algorithm)) {//未知证件
            plainStr = "110103201206200910";
        } else if ("FPE_NUMBER".equals(algorithm)) {
            plainStr = "123.45";
        } else if ("FPE_NUMBER_TWEAK".equals(algorithm)) {
            plainStr = "123.45";
        } else {
            System.out.println("算法不合法");
            return;
        }

        System.out.println("plainData: " + plainStr);

        byte[] plainData = plainStr.getBytes("UTF-8");

        String ivStr = "123456789abcdefg";
        System.out.println("iv: " + ivStr);
        byte[] iv = ivStr.getBytes("UTF-8");

        String metadata = "ABCDBF";
        System.out.println("metadata: " + metadata);

        if ("FPE_NUMBER".equals(algorithm) || "FPE_NUMBER_TWEAK".equals(algorithm)) {
            System.out.println("precision:" + precision);
            System.out.println("scale:" + scale);
        }

        long beginTime = System.currentTimeMillis();

        if ("FPE_NUMBER".equals(algorithm) || "FPE_NUMBER_TWEAK".equals(algorithm)) {
            byte[] cipherData_fpe = CipherSuiteUtils.fpeNumberEncrypt(plainData, algorithm, keyId, metadata, iv, precision, scale);
            System.out.println("encrypt result: " + new String(cipherData_fpe));
            byte[] decryptData_fpe = CipherSuiteUtils.fpeNumberDecrypt(cipherData_fpe, algorithm, keyId, metadata, iv, precision, scale);
            System.out.println("decrypt result: " + new String(decryptData_fpe));
            long endTime = System.currentTimeMillis() - beginTime;
            if (plainStr.equals(new String(decryptData_fpe))) {
                System.out.println("encrypt and decrypt success. time:" + endTime + "ms.");
            }
        } else if ("SM3".equals(algorithm)) {
            byte[] digestData = CipherSuiteUtils.digest(plainData, algorithm, keyId, metadata);
            System.out.println("digest result: " + DatatypeConverter.printHexBinary(digestData));
            long endTime = System.currentTimeMillis() - beginTime;
            System.out.println("SM3 digest success. time:" + endTime + "ms.");
        } else {
            // 加密
            byte[] cipherData = CipherSuiteUtils.encrypt(plainData, algorithm, keyId, metadata, iv);
            if (algorithm.startsWith("FPE")) {
                System.out.println("encrypt result: " + new String(cipherData));
            } else {
                System.out.println("encrypt result: " + DatatypeConverter.printHexBinary(cipherData));
            }

            // 解密
            byte[] decryptData = CipherSuiteUtils.decrypt(cipherData, algorithm, keyId, metadata, iv);
            System.out.println("decrypt result: " + new String(decryptData));

            long endTime = System.currentTimeMillis() - beginTime;
            if (plainStr.equals(new String(decryptData))) {
                System.out.println("encrypt and decrypt success. time:" + endTime + "ms.");
            }
        }

    }
}
